#include "inttypes.h"
#include <avr/io.h>
#include <util/delay.h>
#include "avr/pgmspace.h"

#include "adc.h"
#include "graphics.h"
#include "keys.h"
#include "lcd.h"
#include "main.h"
#include "timmer.h"


const int16_t PROGMEM GAIN_DIVIDER_VALUES[] =
{
	//wzmacniacz 100:1
	2,//50mV
	4,//100mV
	8,//200mV
	//wzmacniacz 10:1
	2,//500mV
	4,//1V
	8,//2V
	//wzmacniacz 1:1
	2,//5V
	4,//10V
	8//20V
};

void AdcInit()
{
	ADCSRA = 	(1<<ADEN) //ADEN=1 włączenie przetwornika ADC)
				|(1<<ADPS1)
				|(1<<ADPS0)
				;// ustawienie preskalera na 4
}

int16_t SetGainDivider(uint8_t gain_base, int16_t adc_for_zero_1_1, int16_t adc_for_zero_1_10, int16_t adc_for_zero_1_100, int16_t *adc_for_zero)
{
	int16_t gain_divider = 1;
	while(ADCSRA & (1<<ADSC));
	if(gain_base > 6)//1:1
	{
		SetAdcActivChannel(ADC_CHANNEL_1_1);
		*adc_for_zero = adc_for_zero_1_1;
	}
	else if(gain_base > 3)//1:10
	{
		SetAdcActivChannel(ADC_CHANNEL_1_10);
		*adc_for_zero = adc_for_zero_1_10;
	}
	else//1:100
	{
		SetAdcActivChannel(ADC_CHANNEL_1_100);
		*adc_for_zero = adc_for_zero_1_100;
	}

	gain_divider = pgm_read_word(&GAIN_DIVIDER_VALUES[gain_base-1]);

	return gain_divider;
}

int16_t GetADCfromChanel(uint8_t channel)
{
	uint8_t prev_ADMUX = ADMUX;
	while(ADCSRA & (1<<ADSC));
	SetAdcActivChannel(channel);
	ADCSRA |= (1<<ADSC);
	while(ADCSRA & (1<<ADSC));
	_delay_ms(10);
	ADMUX = prev_ADMUX;
	return (int16_t)ADC;
}

void SetAdcActivChannel(uint8_t channel)
{
	ADMUX  = 0xc0+channel;
}

int16_t get_mean_ADC_from_channel(uint8_t channel)
{
	uint8_t i = 8;
	uint16_t my_adc  = 0;
	while(i--)
		my_adc += GetADCfromChanel(channel);
	return my_adc/8;
}
