#include <stdio.h>
#include <avr/io.h>
#include <string.h>
#include "inttypes.h"
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "text.h"

#include "adc.h"
#include "graphics.h"
#include "keys.h"
#include "lcd.h"
#include "main.h"
#include "timmer.h"


void inline splash_screen()
{
	lcd_str_3x5(20,0,"PICO SCOPE",0xff,0x80,ON);
	lcd_str_3x5(17,8,"Jan Nawracaj",0xff,0x00,ON);
	lcd_str_3x5(30,24,"2019",0xff,0x00,ON);
	lcd_str_3x5(27,32,"HV 1.0",0xff,0x00,ON);
	lcd_str_3x5(27,40,"SV 1.0",0xff,0x00,ON);
	_delay_ms(2000);
	lcd_clear();
}


const char PROGMEM DIGITS[] =
{
	'0','1','2','3','4','5','6','7','8','9'
};

const uint8_t PROGMEM Y_VALUES[] =
{
	0,4,5
};

void draw_main_window()
{
	uint8_t and[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
	Lcd_line_vert(1, 0, 47, and, and,  and,ON);
	Lcd_line_vert(83, 0, 47, and, and,  and,ON);
	Lcd_line_vert(61, 0, 47, and, and,  and,ON);

	uint8_t j=0x01;
	for(uint8_t i=0;i<3;i++)
	{
		lcd_set_byte(62,pgm_read_byte(&Y_VALUES[i]),j);
		j=0xff;
	}
}

void ChartRefresh(int16_t *data, int16_t gain_divider, int8_t start_index, int8_t vertical_zero, int8_t horizontal_zero, int8_t trigger_level, int8_t ace, int8_t ac1, int8_t ac2, int8_t tce, int8_t tc1, int8_t tc2)
{
	int8_t x=1;
	int8_t next_index=0;
	int8_t v1, v2;

	uint8_t and[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
	uint8_t or_0[6] = {0x0d|1<<5, 1<<2|1<<7, 1<<4, 1<<1|1<<6, 1<<3, 1<<0|1<<5|0x80};
	uint8_t or_1[6] = {0x0d|1<<5, 1<<7, 0, 1<<1, 1<<3, 1<<5|0x80};
	uint8_t or_2[6] = {0x0d|0, 0, 0, 1<<1, 0, 0|0x80};
	uint8_t or_3[6] = {0x0d|0, 0, 0, 0, 0, 0|0x80};
	uint8_t or_tc[6] = {0x0d|0xa0, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa|0x80};
	uint8_t or_ac[6] = {0, 0, 0, 0, 0, 0};

	int8_t buf_start = 30 - horizontal_zero/2 - 15;
	int8_t buf_stop = 30 - horizontal_zero/2 + 15;
	int16_t value = 0;

	uint8_t ac1_byte, ac2_byte, ac1_move, ac2_move;
	int8_t ac_point_status = -1;
	if(ace)
	{
		ac1_byte = ac1/8;
		ac1_move = 1<<(ac1%8);
		ac2_byte = ac2/8;
		ac2_move = 1<<(ac2%8);
	}

	start_index = start_index - 30 - horizontal_zero;
	if(start_index < 0)
		start_index += 120;
	else if(start_index > 119)
		start_index -= 120;

	while(x<60)
	{
		next_index = start_index+1;
		if(next_index > 119)
			next_index = 0;

		value = data[start_index]/gain_divider;
		if(value > 127)
			value = 127;
		else if(value < -127)
			value = -127;
		v1 = 25 - vertical_zero - (int8_t)value;
		if(v1 < 3)
			v1 = 3;
		else if(v1 > 47)
			v1 = 47;
		value = data[next_index]/gain_divider;
		if(value > 127)
			value = 127;
		else if(value < -127)
			value = -127;
		v2 = 25 - vertical_zero - (int8_t)value;
		if(v2 < 3)
			v2 = 3;
		else if(v2 > 47)
			v2 = 47;

		start_index++;
		if(start_index > 119)
			start_index = 0;

		if(x - 30 == horizontal_zero)
			and[0] = 0xf3;
		else if((x - 31 == horizontal_zero) || (x - 29 == horizontal_zero))
			and[0] = 0xfb;
		else
			and[0] = 0xff;

		if(x >= buf_start && x <= buf_stop)
		{
			or_0[0] &=~ 0x02;
			or_1[0] &=~ 0x02;
			or_2[0] &=~ 0x02;
			or_3[0] &=~ 0x02;
			or_tc[0] &=~ 0x02;
		}
		else
		{
			or_0[0] ^= 0x02;
			or_1[0] ^= 0x02;
			or_2[0] ^= 0x02;
			or_3[0] ^= 0x02;
			or_tc[0] ^= 0x02;
		}

		if(ace)
		{
			ac_point_status *= -1;
			if(ac_point_status < 0)
			{
				or_ac[ac1_byte] |= ac1_move;
				or_ac[ac2_byte] |= ac2_move;
			}
			else
			{
				or_ac[ac1_byte] = 0;
				or_ac[ac2_byte] = 0;
			}
		}

		if(tce && (x == tc1 || x == tc2))
			Lcd_line_vert(x+1, v1, v2, and, or_tc, or_ac, ON);
		else if(x==30)
			Lcd_line_vert(x+1, v1, v2, and, or_0, or_ac, ON);
		else if(x%10==0)
			Lcd_line_vert(x+1, v1, v2, and, or_1, or_ac, ON);
		else if(x%5==0)
			Lcd_line_vert(x+1, v1, v2, and, or_2, or_ac, ON);
		else
			Lcd_line_vert(x+1, v1, v2, and, or_3, or_ac, ON);

		x++;

	}


	and[0] = 0xff;
	or_3[0] = 0x0f;
	//SHOW vertical zero
	Lcd_line_vert(0, 25 - vertical_zero, 25 - vertical_zero, and, or_3, or_ac, ON);

	//SHOW TRIGGER LEVEL
	int8_t bit_move = 25 - vertical_zero - trigger_level;
	if(bit_move > 3 && bit_move < 47)
	{
		and[bit_move/8] &=~ (1<<(bit_move%8));
		Lcd_line_vert(1, 0, 47, and, or_3, or_ac, ON);
	}
	else
		Lcd_line_vert(1, 0, 47, and, or_3, or_ac, ON);

}


char get_digit_char(uint8_t digit)
{
	return pgm_read_byte(&DIGITS[digit]);
}

uint8_t pre_prepare_value_buffer(char *buff, uint8_t i, uint16_t val)
{
	uint16_t div = 100;
	uint8_t digit = 0;

	while(div && i < 4)
	{
		digit = val/div;
		if(digit || i)
			buff[i++] = get_digit_char(digit);
		val = val%div;
		div /= 10;
	}
	return i;
}

void prepare_value_buffer(char *buff, uint16_t val1, uint16_t val2, uint16_t val3, char t1, char t2, char t3)
{
	uint16_t div = 100;
	uint8_t digit = 0;
	uint8_t i=0;
	if(val1)
		buff[4] = t1;
	else if(val2)
		buff[4] = t2;
	else if(val3)
		buff[4] = t3;

	i = pre_prepare_value_buffer(buff, i, val1);

	if(i >= 3)
		return;
	else if(i)
		buff[i++] = '.';

	i = pre_prepare_value_buffer(buff, i, val2);

	if(i >= 3)
		return;
	else if(i)
		buff[i++] = '.';

	i = pre_prepare_value_buffer(buff, i, val3);

	if(i >= 3)
		return;

	return;
}

void get_value_from_pix(uint16_t multiplier, uint8_t pix, uint16_t *v2, uint16_t *v1, uint16_t *v0, uint16_t *table)
{
	uint16_t v1_per_pix = pgm_read_word(table)*multiplier;
	uint16_t v0_per_pix = pgm_read_word(table+1)*multiplier;
	while(v0_per_pix > 1000)
	{
		v1_per_pix++;
		v0_per_pix -= 1000;
	}
	uint16_t _v2 = 0;
	uint16_t _v1 = 0;
	uint16_t _v0 = 0;

	uint8_t i = 0;
	for(;i<pix;i++)
	{
		_v0 += v0_per_pix;
		if(_v0 >= 1000)
		{
			_v1++;
			_v0 -= 1000;
		}
		_v1 += v1_per_pix;
		if(_v1 >= 1000)
		{
			_v2++;
			_v2 -= 1000;
		}
	}
	*v2 = _v2;
	*v1 = _v1;
	*v0 = _v0;
}

const uint16_t PROGMEM VOLTAGE_PER_PIX[][2] =
{
//	V,mV
	{0,5},
	{0,10},
	{0,20},
	{0,50},
	{0,100},
	{0,200},
	{0,500},
	{1,0},
	{2,0}
};

const uint16_t PROGMEM TIME_PER_PIX[][2] =
{
//	ms,us
	{0,20},//"200us",
	{0,50},//"500us",
	{0,100},//"  1ms",
	{0,200},//"  2ms",
	{0,500},//"  5ms",
	{1,0},//" 10ms",
	{2,0},//" 20ms",
	{5,0},//" 50ms",
	{10,0},//"100ms",
	{20,0},//"200ms",
	{50,0}//"500ms"
};

void refresh_delta_t(uint8_t time_base, int8_t c1, int8_t c2)
{
	uint16_t s, ms, us;
	get_value_from_pix(1, abs(c1 - c2), &s, &ms, &us, &TIME_PER_PIX[time_base-2][0]);
	char buff[6] = "     ";
	prepare_value_buffer(buff,s,ms,us,'s','m','u');
	buff[5] = 0;
	lcd_str_3x5(63, 40, buff, 0xff, 0x00, OFF);
}

void refresh_delta_V(uint8_t gain_base, uint16_t probe, int8_t c1, int8_t c2)
{
	uint16_t k, V, m;
	get_value_from_pix(probe, abs(c1 - c2), &k, &V, &m, &VOLTAGE_PER_PIX[gain_base-1][0]);
	char buff[6] = "     ";
	prepare_value_buffer(buff,k,V,m,'k','V','m');
	buff[5] = 0;
	lcd_str_3x5(63, 32, buff, 0xff, 0x00, OFF);
}

void refresh_time_base(uint8_t time_base)
{
	char buff[6];
	read_text_from_pgm(&TIME_VALUE_TEXT[time_base-2], buff, 5);
	lcd_str_3x5(63, 40, buff, 0xff, 0x00, OFF);
}

void refresh_gain(uint8_t gain_base, uint16_t probe_divider)
{
	if(probe_divider == 100)
		gain_base += 6;
	else if(probe_divider == 10)
		gain_base += 3;

	char buff[6];

	read_text_from_pgm(&GAIN_VALUE_TEXT[gain_base-1], buff, 5);
	lcd_str_3x5(63, 32, buff, 0xff, 0x00, OFF);
}

void show_menu_cursor(uint8_t current_index)
{
	int8_t y=0;
	uint8_t flag = 0x01;
	for(uint8_t i = 0; i < 4; i++)
	{
		if(i == current_index || (i == 3 && current_index > 3))
			lcd_str_3x5(63, y, ">", 0xff, flag, ON);
		else
			lcd_str_3x5(63, y, " ", 0xff, flag, ON);
		y+=8;
		flag = 0;
	}
}

void show_main_menu(int8_t current_index)
{
	show_menu_text(current_index, 255, MAIN_MENU_TEXT);
}

void show_trigger_menu(uint8_t current_index)
{
	show_menu_text(current_index, 255, TRIGGER_MENU_TEXT);
}

void read_text_from_pgm(const unsigned char *addr, char *buff, uint8_t l)
{
	uint8_t i = 0;
	for(i = 0; i < l; i++)
		buff[i] = pgm_read_byte(&addr[i]);
	buff[i] = 0;
}

void show_trigger_mode(uint8_t current_index, trigg_mode trigger_mode)
{
	show_menu_text(current_index, trigger_mode, TRIGGER_MODE_TEXT);
}

void show_cursors(uint8_t current_index)
{
	show_menu_text(current_index, 255, CURSORS_TEXT);
}

void show_menu_text(uint8_t current_index, uint8_t activ, const unsigned char *addr)
{

	uint8_t y_start = 0;
	uint8_t top = 0x01;
	char text_buff[5];
	int8_t i = current_index - 3;
	if(i < 0)
		i = 0;

	addr += 4*i;
	show_menu_cursor(current_index);
	for(uint8_t j = 0; j < 4; j++)
	{
		read_text_from_pgm(addr, text_buff, 4);
		if(i + j == activ)
		{
			lcd_str_3x5(67, y_start, text_buff, 0xff, top, OFF);
			lcd_set_byte(66, y_start/8, 0xff);
		}
		else
		{
			lcd_str_3x5(67, y_start, text_buff, 0xff, top, ON);
			lcd_set_byte(66, y_start/8, top);
		}
		top = 0;
		y_start += 8;
		addr = addr + 4;
	}
}

void show_trigger_type(uint8_t current_index, trigg_type trigger_type)
{
	show_menu_text(current_index, trigger_type, TRIGGER_TYPE_TEXT);
}

void show_back_only()
{
	show_menu_text(0, 255, BACK_ONLY_TEXT);
}

void show_probe_divider(uint8_t current_index, uint16_t probe_divider)
{
	uint8_t activ = 0;
	while(probe_divider > 1)
	{
		probe_divider /= 10;
		activ++;
	}
	show_menu_text(current_index, activ, PROBE_DIVIDER_TEXT);
}

const uint8_t PROGMEM BATTERY_ICON_TABLE[] =
{
		1,1,1,0x7d,0x45,0x45,0x45,0x45,0x45,0x45,0x45,0x45,0x45,0x45,0x45,0x7d,0x39,1,1,1
};

void show_battery_lvl(int16_t bat_adc, int16_t min, int16_t max)
{

	//Przerabia pomiar na wartość 0-10
	if(bat_adc < min)
		bat_adc = min;
	else if(bat_adc > max)
		bat_adc = max;

	bat_adc -= min;
	max -= min;
	bat_adc = (bat_adc * 10)/max;

	for(int8_t i=0;i<20;i++)
	{
		if(i>=4 && i-4 <= bat_adc)
			lcd_set_byte(63+i,0,0x7d);
		else
			lcd_set_byte(63+i,0,pgm_read_byte(&BATTERY_ICON_TABLE[i]));
	}
}

const uint8_t PROGMEM TRIGGER_TYPE_ICON_TABLE[] =
{
		0x90,0x98,0xff,0x19,0x11,0x00,0x09,0x19,0xff,0x98,0x88,0x00
};

void show_trigger_type_mode_state(trigg_mode trigger_mode, trigg_type trigger_type, uint8_t trigger_state)
{
	uint8_t i = trigger_type*6;
	uint8_t j=0;
	for(;j<6;j++)
		lcd_set_byte(63+j,1,pgm_read_byte(&TRIGGER_TYPE_ICON_TABLE[i++]));

	lcd_set_byte(63+j,1,0x00);

	char text_buff[4];
	text_buff[3]=0;
	read_text_from_pgm(TRIGGER_MODE_TEXT_SH0RT[trigger_mode], text_buff, 3);
	lcd_str_3x5(70, 8, text_buff, 0xff, 0x00, ON);
}

void show_normal_loop_values(trigg_mode trigger_mode, trigg_type trigger_type, uint8_t trigger_state)
{
	show_trigger_type_mode_state(trigger_mode, trigger_type, trigger_state);

	for(uint8_t i=0;i<20;i++)
	{
		lcd_set_byte(63+i,2,0);
		lcd_set_byte(63+i,3,0);
	}
}
