/*
** lcd.c
**
** LCD 3310 driver
** Unbuffered version - very small memory footprint
** Target: ATMEGA128 :: AVR-GCC
**
** Written by Tony Myatt - 2007
** Quantum Torque - www.quantumtorque.com
*/
#include <stdio.h>
#include <avr/io.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "avr/eeprom.h"

#include "fonts.h"

#include "adc.h"
#include "graphics.h"
#include "keys.h"
#include "lcd.h"
#include "main.h"
#include "timmer.h"


//Bufor pamiêci RAM
//uint8_t  CACHE_LCD[LCD_CACHE_SIZE];

//indeks wirtualnej pamiêci CACHE_LCD
uint16_t CACHE_LCD_Idx;

/* makrodefinicje */
//Czas Resetu
#define reset_delay() for(int i=-32000;i<32000;i++)

uint16_t CACHE_LCD_Idx;

const uint8_t PROGMEM LCD_INIT_COMMANDS[] =
{
	    0x21,// LCD Extended Commands
		0xbf,// Set LCD Vop(Contrast)
		0x06,// Set Temp coefficent
		0x13,// LCD bias mode 1:48
		0x20,// Standard Commands, Horizontal addressing
		0x0C// LCD in normal mode
};

void lcd_init(void)
{
	DDRD |= 0x1f;
    RST_H();
    _delay_ms(15);
    RST_L();
    reset_delay();
    RST_H();
    // Disable LCD controller
    CE_H();

    for(uint8_t i=0;i<6;i++)
    	lcd_send(pgm_read_byte(&LCD_INIT_COMMANDS[i]), CMD);
}

//Funkcja czyszcz¹ca ekran
void lcd_clear(void)
{
    for(uint16_t i=0;i<LCD_CACHE_SIZE;i++)
    	lcd_send(0, DAT);
}


//Funkcja ustawiaj¹ca adres do wys³ania danych
void lcd_addres(uint16_t addr)
{
	lcd_send(0x80 |(addr % X_RES), CMD);//numer bajtu w wierszu
	lcd_send(0x40 |(addr / X_RES), CMD);//numer wiersza
}

//Ustawia kursor na pozycji x y w buforze
void lcd_xy(uint8_t x, uint8_t y)
{
    CACHE_LCD_Idx = (x) + (y/8)*X_RES;
}
void lcd_char_3x5( char chr, uint8_t mask_and, uint8_t mask_or, pixels_mode mode)
{
    for(unsigned char i=0;i<4;i++)
	{
    	if(mode == ON)
    		lcd_send(((pgm_read_byte(&font3x5[chr-32][i])<<2)&mask_and)|mask_or, DAT);
    	else
    		lcd_send(~(((pgm_read_byte(&font3x5[chr-32][i])<<2)&mask_and))|mask_or, DAT);
		CACHE_LCD_Idx += 4;
	}

}

/* Displays string at current cursor location and increment cursor location */
void lcd_str_3x5(uint8_t x, uint8_t y, char *str, uint8_t mask_and, uint8_t mask_or, pixels_mode mode)
{
	lcd_xy(x,y);
    lcd_addres(CACHE_LCD_Idx);
    while(*str)
	lcd_char_3x5(*str++, mask_and, mask_or, mode);
}


void Lcd_line_vert(int8_t x, int8_t y1, int8_t y2, uint8_t *mask_and, uint8_t *mask_or, uint8_t *mask_or2, pixels_mode mode)
{
    int8_t yp, yk;
    uint8_t data = 0;
    if(y2 >= y1)
    {
    	yp = y1;
    	yk = y2;
    }
    else
    {
    	yk = y1;
    	yp = y2;
    }
    uint8_t bit = 0;
    uint8_t bajt = 0;
    uint8_t y = 0;
    while(y <= 47 )
    {
    	bit = y%8;
		bajt = y/8;
		if(y >= yp && y <= yk)
		{
			if(mode)
				data |= 1<<bit;
		}
		if(bit == 7)
		{
			lcd_xy(x,y);
			lcd_addres(CACHE_LCD_Idx);
			lcd_send((data | mask_or[bajt]| mask_or2[bajt]) & mask_and[bajt], DAT);
			data = 0;
		}
		y++;
    }
}

void lcd_set_byte(int8_t x, int8_t y, uint8_t byte)
{
	lcd_addres(x + y*X_RES);
	lcd_send(byte, DAT);
}


void lcd_send(unsigned char data, uint8_t cd)
{
	CE_L();
    if(cd == 1)
        DC_H();
    else
        DC_L();

	for(unsigned char i=0;i<8;i++)
	{
		// Set the DATA pin value
		if((data>>(7-i)) & 0x01)
			DATA_H();
		else
			DATA_L();

		// Toggle the clock
		CLK_H();
		CLK_L();
	}
	// Disable display controller
    CE_H();
}

