#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>
#include "avr/interrupt.h"
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/sleep.h>
#include "inttypes.h"
#include "stdint.h"


#include "adc.h"
#include "graphics.h"
#include "keys.h"
#include "lcd.h"
#include "main.h"
#include "timmer.h"


volatile uint16_t timer1_cnt_start = 0;
volatile int8_t adc_buff_index=0;
volatile int8_t adc_data_index = 0;
volatile int16_t adc_buff[120];
volatile int16_t adc_buff_work[120];
volatile uint8_t _20_ms_cnt = 0;
volatile int8_t trigger_index = 60;
volatile uint8_t sampling = 0;
volatile int16_t pr_ADC = 0;
volatile uint8_t past_trigger_counter = 0;
volatile int8_t exp_trigger = 0;
volatile uint8_t force_trigger = 0;
volatile uint8_t _20ms_no_trigger = 0;
volatile uint8_t trigger_one_execute = 0;


int8_t ac1 = 15;
int8_t ac2 = 35;
int8_t tc1 = 10;
int8_t tc2 = 50;
int8_t ace = 0;
int8_t tce = 0;

int16_t battery_lvl = 0;

int8_t vertical_zero = 0;
int8_t horizontal_zero = 0;
uint8_t time_base = 2;
uint8_t gain_base = 5;
int16_t adc_for_zero_1_1 = 512;
int16_t adc_for_zero_1_10 = 512;
int16_t adc_for_zero_1_100 = 512;
int16_t adc_for_zero = 512;
uint16_t probe_divider = 1;
int16_t gain_divider = 1;
int8_t trigger_level = 0;
int16_t ADC_trigger_level = 0;
uint16_t mv_2lsb = 5;

l_mode loop_mode = LOOP_NORMAL;

uint8_t main_menu_current_index = 0;
uint8_t trig_menu_current_index = 0;
uint8_t trig_mode_menu_current_index = 0;
uint8_t trig_type_menu_current_index = 0;
uint8_t curs_menu_current_index = 0;
uint8_t probe_menu_current_index = 0;

trigg_mode trigger_mode = TM_AUTO;
trigg_type trigger_type = TT_RSLOPE;


//Eeprom
static EEMEM int8_t EE_vertical_zero;
static EEMEM int8_t EE_horizontal_zero;
static EEMEM uint8_t EE_time_base;
static EEMEM uint8_t EE_gain_base;
static EEMEM int16_t EE_adc_for_zero_1_1;
static EEMEM int16_t EE_adc_for_zero_1_10;
static EEMEM int16_t EE_adc_for_zero_1_100;
static EEMEM uint16_t EE_probe_divider;
static EEMEM int8_t EE_trigger_level;
static EEMEM trigg_mode EE_trigger_mode;
static EEMEM trigg_type EE_trigger_type;

const uint16_t PROGMEM LOOPS_FUNCTIONS_POINTERS[] =
{
	&normal_loop, &main_menu_loop, &gain_loop, &time_loop, &vertical_pos_loop, &horizontal_pos_loop, &trigger_menu_loop,
	&trigger_mode_loop, &trigger_type_loop, &trigger_level_loop, &cursors_loop, &AC1_loop, &AC2_loop, &TC1_loop, &TC2_loop, &probe_loop
};

void main(void)
{
	void (*loop_pointer)();
	VCC_DDR |= (1<<VCC_PIN_NUMBER);
	VCC_PORT |= (1<<VCC_PIN_NUMBER);
	_delay_ms(10);
	battery_lvl = get_mean_ADC_from_channel(ADC_CHANNEL_BAT);
	sampling = 1;
	lcd_init();//inicjalizacja LCD
	lcd_clear();
	splash_screen();
	draw_main_window();
	load_set();
	AdcInit();
	SampleTimerInit();
	KeysInit();
	MainTimerInit();
	timer1_cnt_start = SampleTimerSetFreq(time_base);
	refresh_time_base(time_base);
	gain_divider = SetGainDivider(gain_base, adc_for_zero_1_1, adc_for_zero_1_10, adc_for_zero_1_100, &adc_for_zero);
	refresh_gain(gain_base, probe_divider);
	ADC_trigger_level = (int16_t)trigger_level * (int16_t)gain_divider;

	while(1)
	{
		loop_pointer = pgm_read_word(&LOOPS_FUNCTIONS_POINTERS[loop_mode]);
		loop_pointer();

		if(sampling == 0)
		{
			if(_20_ms_cnt == 1)
			{
				battery_lvl = get_mean_ADC_from_channel(ADC_CHANNEL_BAT);
				_20_ms_cnt = 0;
			}

			for(uint8_t l=0;l<120;l++)
				adc_buff_work[l] = adc_buff[l];
			past_trigger_counter = 0;
			sampling = 1;
		}
		ChartRefresh(adc_buff_work, gain_divider, trigger_index, vertical_zero, horizontal_zero, trigger_level, ace, ac1, ac2, tce, tc1, tc2);
	}
}

//Timer 1 próbkowanie ADC
ISR(TIMER1_OVF_vect)
{
	TCNT1 = timer1_cnt_start;
	int16_t MY_ADC = (int16_t)ADC - adc_for_zero;
	ADCSRA |= (1<<ADSC); //ADSC: uruchomienie pojedynczej konwersji
	if(sampling)
	{
		adc_buff[adc_data_index] = MY_ADC;

		if(trigger_mode == TM_NORMAL || trigger_mode == TM_AUTO || (trigger_mode == TM_ONE && trigger_one_execute))
		{
			if(trigger_type == TT_FSLOPE)
			{
				if(pr_ADC > ADC_trigger_level && MY_ADC < ADC_trigger_level)
				{
					if(exp_trigger == 1)
						force_trigger = 1;
					_20ms_no_trigger = 0;

					trigger_one_execute = 0;
				}
			}
			else if(trigger_type == TT_RSLOPE)
			{
				if(pr_ADC < ADC_trigger_level && MY_ADC > ADC_trigger_level)
				{
					if(exp_trigger == 1)
						force_trigger = 1;
					_20ms_no_trigger = 0;

					trigger_one_execute = 0;
				}
			}
		}

		if(exp_trigger == 0)
		{
			if(past_trigger_counter < 60)
				past_trigger_counter++;
			else
				exp_trigger = 1;
		}
		else if(exp_trigger == 1)
		{
			if(trigger_mode == TM_AUTO)
			{
				if(_20ms_no_trigger > 50)
					force_trigger = 1;
			}

			if(force_trigger && exp_trigger == 1)
			{
				force_trigger = 0;
				trigger_index = adc_data_index;
				exp_trigger = 2;
			}
		}
		else if(exp_trigger == 2)
		{
			if(past_trigger_counter < 120)
				past_trigger_counter++;
			else
			{
				sampling = 0;
				exp_trigger = 0;
			}
		}

		if(adc_data_index < 119)
			adc_data_index++;
		else
			adc_data_index = 0;
	}
	pr_ADC = MY_ADC;
}


//Timer 0 co 20 ms
ISR(TIMER0_OVF_vect)
{
   TCNT0 = 100;
   KeysRead();
   if(trigger_mode == TM_AUTO)
   {
	   if(_20ms_no_trigger < 255)
		   _20ms_no_trigger++;
   }
   if(_20_ms_cnt>1)
	   _20_ms_cnt--;
}

const uint16_t PROGMEM RAM_SETTINGS_ADRESS[] =
{
		&vertical_zero,&horizontal_zero,&time_base,&gain_base,&adc_for_zero_1_1,&adc_for_zero_1_10,&adc_for_zero_1_100,&probe_divider,&trigger_level,&trigger_mode,&trigger_type
};

const uint16_t PROGMEM EEPROM_SETTINGS_ADRESS[] =
{
		&EE_vertical_zero,&EE_horizontal_zero,&EE_time_base,&EE_gain_base,&EE_adc_for_zero_1_1,&EE_adc_for_zero_1_10,&EE_adc_for_zero_1_100,&EE_probe_divider,&EE_trigger_level,&EE_trigger_mode,&EE_trigger_type
};

const uint8_t PROGMEM EEPROM_DATA_SIZES[] =
{
	1,1,1,1,2,2,2,2,1,1,1
};
void save_set()
{
	for(uint8_t i=0;i<11;i++)
		eeprom_write_block(pgm_read_word(&RAM_SETTINGS_ADRESS[i]), pgm_read_word(&EEPROM_SETTINGS_ADRESS[i]), pgm_read_byte(&EEPROM_DATA_SIZES[i]));
}

void load_set()
{
	for(uint8_t i=0;i<11;i++)
		eeprom_read_block(pgm_read_word(&RAM_SETTINGS_ADRESS[i]), pgm_read_word(&EEPROM_SETTINGS_ADRESS[i]), pgm_read_byte(&EEPROM_DATA_SIZES[i]));
}

const uint8_t PROGMEM MAIN_MENU_LOOP_MODES[] =
{
		LOOP_GAIN,LOOP_TIME,LOOP_VERTICAL_POS,LOOP_HORIZONTAL_POS,LOOP_TRIGG_MENU,LOOP_CURSORS,LOOP_PROBE,255,255,255,LOOP_NORMAL
};

void (*resetptr)( void ) = 0x0000;

void main_menu_loop()
{
	uint8_t keys = keys_increment_decrement(0, 10, &main_menu_current_index, 1,1);
	show_main_menu(main_menu_current_index);

	if(keys & KEY3)
	{
		if(main_menu_current_index==7)//kaliracja
		{
			TCCR1B = 0;
			adc_for_zero_1_1  = get_mean_ADC_from_channel(ADC_CHANNEL_1_1);
			adc_for_zero_1_10  = get_mean_ADC_from_channel(ADC_CHANNEL_1_10);
			adc_for_zero_1_100  = get_mean_ADC_from_channel(ADC_CHANNEL_1_100);
			gain_divider = SetGainDivider(gain_base, adc_for_zero_1_1, adc_for_zero_1_10, adc_for_zero_1_100, &adc_for_zero);
			TCCR1B |= (1<<CS11);//clk/8
		}
		else if(main_menu_current_index==8)//zapis ustawień
			save_set();
		else if(main_menu_current_index==9)//Wyłączenie
			turn_off();
		else
		{
			if(main_menu_current_index==5)//Cursors
			{
				ace = 1;
				tce = 1;
				refresh_delta_V(gain_base, probe_divider, ac1, ac2);
				refresh_delta_t(time_base, tc1, tc2);
			}
			loop_mode = pgm_read_byte(&MAIN_MENU_LOOP_MODES[main_menu_current_index]);
		}
	}
}

void inline turn_off()
{
	DDRB = 0;
	DDRC = 0;
	DDRD = 0;
	PORTB = 0;
	PORTC = 0;
	PORTD = 0;
	VCC_DDR |= (1<<VCC_PIN_NUMBER);
	TCCR0B = 0;
	TCCR1B = 0;
	ADCSRA = 0;

	_delay_ms(500);

	SREG |= 1<<SREG_I;
	PCICR |= 1<<PCIE0;//zmiana na pinach PCINT7..0 powoduje PCI0 vektor
	PCMSK0 |= 1<<PCINT1;//PCINT1

	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	sleep_enable();
	sleep_cpu();

	sleep_disable();
	cli();
	loop_mode = LOOP_NORMAL;
	SREG = 0;
	PCICR = 0;
	PCMSK1 = 0;
	_delay_ms(500);
	resetptr();
}
void gain_loop()
{
	uint8_t keys = keys_increment_decrement(1, 9, &gain_base, 0,1);
	show_back_only();

	uint8_t gain_change = 0;
	if(keys & KEY2 || keys & KEY1)
		gain_change = 1;

	if(gain_change)
	{
		refresh_gain(gain_base, probe_divider);
		gain_divider = SetGainDivider(gain_base, adc_for_zero_1_1, adc_for_zero_1_10, adc_for_zero_1_100, &adc_for_zero);
		ADC_trigger_level = (int16_t)trigger_level * (int16_t)gain_divider;
	}

	if(keys & KEY3)
	{
		loop_mode = LOOP_MAIN_MENU;
	}
}

void time_loop()
{
	uint8_t keys = keys_increment_decrement(2, 12, &time_base, 0,-1);
	show_back_only();

	if(keys & KEY3)
		loop_mode = LOOP_MAIN_MENU;

	timer1_cnt_start = SampleTimerSetFreq(time_base);
	refresh_time_base(time_base);
}

const uint8_t PROGMEM TRIGGER_MENU_LOOP_MODES[] =
{
		LOOP_TRIGG_MENU_MODE,LOOP_TRIGG_MENU_TYPE,LOOP_TRIGG_MENU_LEVL,0,LOOP_MAIN_MENU
};

void trigger_menu_loop()
{
	uint8_t keys = keys_increment_decrement(0, 4, &trig_menu_current_index, 1,1);
	show_trigger_menu(trig_menu_current_index);

	if(keys & KEY3)
	{
		if(trig_menu_current_index==3)//Trigger execute
		{
			if(trigger_mode == TM_ONE)
				trigger_one_execute = 1;
			else if(trigger_mode == TM_MANUAL)
				force_trigger = 1;
		}
		else
			loop_mode = pgm_read_byte(&TRIGGER_MENU_LOOP_MODES[trig_menu_current_index]);
	}
}

const uint8_t PROGMEM TRIGGER_MODES[] =
{
		TM_NORMAL,TM_AUTO,TM_MANUAL,TM_ONE,TM_OFF
};
void trigger_mode_loop()
{
	uint8_t keys = keys_increment_decrement(0, 5, &trig_mode_menu_current_index, 1,1);
	show_trigger_mode(trig_mode_menu_current_index, trigger_mode);

	if(keys & KEY3)
	{
		if(trig_mode_menu_current_index==5)//back
		{
			loop_mode = LOOP_TRIGG_MENU;
			trig_mode_menu_current_index = 0;
		}
		else
			trigger_mode = pgm_read_byte(&TRIGGER_MODES[trig_mode_menu_current_index]);
	}
}

void AC1_loop()
{
	uint8_t keys = keys_increment_decrement(4, 46, &ac1, 0,1);
	show_back_only();

	if(keys & KEY3)
		loop_mode = LOOP_CURSORS;
	refresh_delta_V(gain_base, probe_divider, ac1, ac2);
}

void AC2_loop()
{
	uint8_t keys = keys_increment_decrement(4, 46, &ac2, 0,1);
	show_back_only();

	if(keys & KEY3)
		loop_mode = LOOP_CURSORS;

	refresh_delta_V(gain_base, probe_divider, ac1, ac2);
}

void TC1_loop()
{
	uint8_t keys = keys_increment_decrement(1, 59, &tc1, 0,-1);
	show_back_only();

	if(keys & KEY3)
		loop_mode = LOOP_CURSORS;

	refresh_delta_t(time_base, tc1, tc2);
}

void TC2_loop()
{
	uint8_t keys = keys_increment_decrement(1, 59, &tc2, 0,-1);
	show_back_only();

	if(keys & KEY3)
		loop_mode = LOOP_CURSORS;

	refresh_delta_t(time_base, tc1, tc2);
}

const uint8_t PROGMEM CURSOR_LOOP_MODES[] =
{
		LOOP_AC1, LOOP_AC2, LOOP_TC1, LOOP_TC2, LOOP_MAIN_MENU
};

void cursors_loop()
{
	uint8_t keys = keys_increment_decrement(0, 4, &curs_menu_current_index, 1,1);
	show_cursors(curs_menu_current_index);

	if(keys & KEY3)
	{
		loop_mode = pgm_read_byte(&CURSOR_LOOP_MODES[curs_menu_current_index]);
		if(curs_menu_current_index==4)//back
		{
			curs_menu_current_index = 0;
			ace = 0;
			tce = 0;
			refresh_time_base(time_base);
			refresh_gain(gain_base, probe_divider);
		}
	}
}

void trigger_type_loop()
{
	uint8_t keys = keys_increment_decrement(0, 2, &trig_type_menu_current_index, 1,1);
	show_trigger_type(trig_type_menu_current_index, trigger_type);

	if(keys & KEY3)
	{
		if(trig_type_menu_current_index==2)//back
		{
			loop_mode = LOOP_TRIGG_MENU;
			trig_type_menu_current_index = 0;
		}
		else if(trig_type_menu_current_index==0)
			trigger_type = TT_RSLOPE;
		else if(trig_type_menu_current_index==1)
			trigger_type = TT_FSLOPE;
	}
}

int8_t trigger_level_check(int8_t vertical_zero2, int8_t trigger_level2)
{
	if(25 - vertical_zero2 - trigger_level2 < 4)
		trigger_level2 = 25 - vertical_zero2 - 4;
	if(25 - vertical_zero2 - trigger_level2 > 46)
		trigger_level2 = 25 - vertical_zero2 - 46;
	return trigger_level2;
}

void trigger_level_loop()
{
	uint8_t keys = keys_increment_decrement(-25, 25, &trigger_level, 0,-1);
	show_back_only();
	trigger_level = trigger_level_check(vertical_zero, trigger_level);

	if(keys & KEY3)
		loop_mode = LOOP_TRIGG_MENU;
	ADC_trigger_level = (int16_t)trigger_level * (int16_t)gain_divider;
}

const uint8_t PROGMEM PROBE_DIVIDERS_SET_TABLE[] =
{
		1,10,100
};
void probe_loop()
{
	uint8_t keys = keys_increment_decrement(0, 3, &probe_menu_current_index, 1,1);
	show_probe_divider(probe_menu_current_index, probe_divider);

	if(keys & KEY3)
	{
		if(probe_menu_current_index == 3)
		{
			loop_mode = LOOP_MAIN_MENU;
			probe_menu_current_index = 0;
		}
		else
		{
			probe_divider = (uint16_t)pgm_read_byte(&PROBE_DIVIDERS_SET_TABLE[probe_menu_current_index]);
			refresh_gain(gain_base, probe_divider);
		}
	}
}

void vertical_pos_loop()
{
	uint8_t keys = keys_increment_decrement(-21, 21, &vertical_zero, 0,-1);
	show_back_only();
	trigger_level = trigger_level_check(vertical_zero, trigger_level);

	if(keys & KEY3)
		loop_mode = LOOP_MAIN_MENU;

	ADC_trigger_level = (int16_t)trigger_level * (int16_t)gain_divider;
}

void horizontal_pos_loop()
{
	uint8_t keys = keys_increment_decrement(-29, 29, &horizontal_zero, 0,-1);
	show_back_only();

	if(keys & KEY3)
		loop_mode = LOOP_MAIN_MENU;
}
void normal_loop()
{
	if(_20_ms_cnt == 0)//max co 1 s
	{
		show_battery_lvl(battery_lvl, (int)ADC_BAT_VALUE_MIN, (int)ADC_BAT_VALUE_MAX);
		_20_ms_cnt = 100;
	}
	uint8_t keys = KeysGet();
	show_normal_loop_values(trigger_mode, trigger_type, exp_trigger);

	if(keys & KEY3)
	{
		loop_mode = LOOP_MAIN_MENU;
		main_menu_current_index = 0;
	}
}

ISR(PCINT0_vect)
{

}
